# DEMO: Installing the Microsoft Graph PowerShell SDK

#region Install the Microsoft Graph PowerShell SDK

# PowerShell SDK for Microsoft Graph GitHub repository
start 'https://github.com/microsoftgraph/msgraph-sdk-powershell'

# Microsoft Graph PowerShell module
start 'https://www.powershellgallery.com/packages/Microsoft.Graph'
start 'https://www.powershellgallery.com/packages/Microsoft.Graph.Beta'

# Graph Explorer
start 'https://aka.ms/ge'

Find-Module Microsoft.Graph
Find-Module Microsoft.Graph.Beta

# Installing the main module of the SDK will install all 40 sub-modules.
# Consider only installing the necessary modules, including Microsoft.Graph.Authentication.

# You can install the SDK in PowerShell 7 or Windows PowerShell
# My recommendation is to install v2 of the SDK in PowerShell 7
# and v1 of the SDK in Windows PowerShell

# Install v2 of the SDK in PowerShell 7
# pwsh
# For Microsoft Graph v1.0
Install-Module Microsoft.Graph -Scope CurrentUser -Verbose -AllowClobber
# For Microsoft Graph beta
Install-Module Microsoft.Graph.Beta -Scope CurrentUser -Verbose -AllowClobber

# Install v1 of the SDK in Windows PowerShell
# powershell
Install-Module Microsoft.Graph -Scope CurrentUser -Verbose -AllowClobber


# Verify the installation
Get-Module Microsoft.Graph -ListAvailable
Get-InstalledModule Microsoft.Graph

# Updating the SDK
Update-Module Microsoft.Graph

# Uninstalling the SDK

# Uninstall the main module
Uninstall-Module Microsoft.Graph
# Remove all of the dependency modules
Get-InstalledModule Microsoft.Graph.* | ForEach-Object { if ($_.Name -ne "Microsoft.Graph.Authentication") { Uninstall-Module $_.Name } }
Uninstall-Module Microsoft.Graph.Authentication

#endregion

#region API version

# With v1
# By default, the SDK uses the Microsoft Graph REST API v1.0.
# You can change this by using the Select-MgProfile command.
Get-MgProfile
# Select-MgProfile -Name beta 

Connect-MgGraph
Select-MgProfile beta
$BetaUsers = Get-MgUser
Select-MgProfile v1.0
$V1Users = Get-MgUser

# With v2
Connect-MgGraph
$BetaUsers = Get-MgBetaUser
$V1Users = Get-MgUser

#endregion

#region Finding commands

Get-Module microsoft.graph.* -ListAvailable
Get-Module microsoft.graph.* -ListAvailable | Measure-Object

Get-Command -Module Microsoft.Graph.Authentication |
Sort-Object noun |
Format-Table -GroupBy noun

Get-Command -Module Microsoft.Graph.Users |
Sort-Object noun |
Format-Table -GroupBy noun

#endregion