
#region DEMO: Finding Microsoft Graph commands and permissions

#region Finding Microsoft Graph PowerShell commands

Get-Command Find-MgGraphCommand -Syntax

# Find Microsoft Graph PowerShell commands by URI
Find-MgGraphCommand -Uri '/users/{id}'
Find-MgGraphCommand -Uri '/users/{id}' -Method GET -ApiVersion v1.0

# Find Microsoft Graph PowerShell commands by command name
Find-MgGraphCommand -Command 'New-MgUser'
Find-MgGraphCommand -Command 'Get-MgUser'

# Find Microsoft Graph PowerShell commands using a command wildcard
Find-MgGraphCommand -Command *MgUser* -APIVersion 'v1.0'
Find-MgGraphCommand -Command .*MgUser.* -APIVersion 'v1.0'

# Find Microsoft Graph PowerShell commands using a URI wildcard
Find-MgGraphCommand -Uri *users* -Method 'Get' -ApiVersion 'v1.0'
Find-MgGraphCommand -Uri ".*users.*" -Method 'Get' -ApiVersion 'v1.0'


Find-MgGraphCommand -Command Get-MgGroup | fl *
Find-MgGraphCommand -Uri /groups -Method GET -ApiVersion 'v1.0' | Select-Object -ExpandProperty Permissions
# Permissions (from least to most privileged)
start https://learn.microsoft.com/en-us/graph/api/group-list?view=graph-rest-1.0&tabs=http#permissions

# It looks like Find-MgGraphCommand returns only delegated permissions
# When we compare the following output with https://learn.microsoft.com/en-us/graph/api/group-post-groups?view=graph-rest-1.0&tabs=powershell#permissions
# we see that there is one more application permission: Group.Create
Find-MgGraphCommand -Uri /groups -Method POST

#endregion

#region Finding permissions

Get-Command Find-MgGraphPermission -Syntax 

# Find permissions related to a given domain
Find-MgGraphPermission -SearchString application # -PermissionType Application/Delegated

# Surprise!
# Get-Help Find-MgGraphPermission -Online
Get-Help Find-MgGraphPermission -Full
start https://docs.microsoft.com/graph/permissions-reference
# Microsoft Graph Permission Explorer
start https://graphpermissions.merill.net/

# Find the identifier for a specific permission
Find-MgGraphPermission Application.Read | Format-List

'Application.Read.All' | Find-MgGraphPermission -ExactMatch -PermissionType Application | Format-List

Find-MgGraphPermission -All | Measure-Object
# All:         514
# Application: 222
# Delegated:   292

Get-Help Find-MgGraphPermission -Parameter Online
# Connect-MgGraph
Find-MgGraphPermission -Online
Find-MgGraphPermission -All | Measure-Object
# All:         825
# Application: 384
# Delegated:   441

#endregion

#endregion