#region Configuring app-only access for a simple script to list users and groups in your Microsoft 365 tenant

# Certificate

# You will need an X.509 certificate installed in your user's trusted store on the machine where you will run the script.
# You'll also need the certificate's public key exported in .cer, .pem, or .crt format.
# You'll need the value of the certificate subject or its thumbprint.

# Create and export your public certificate without a private key

# Use this method to authenticate from an application running from your machine
# For example, authenticate from PowerShell 7 or Windows PowerShell

# The following command worked in Windows PowerShell
$cert = New-SelfSignedCertificate -Subject "CN=PowerShellTrainingCert" -CertStoreLocation "Cert:\CurrentUser\My" -KeyExportPolicy Exportable -KeySpec Signature -KeyLength 2048 -KeyAlgorithm RSA -HashAlgorithm SHA256
# The following failed in PowerShell 7
# $cert = New-SelfSignedCertificate -SubjectName "CN=PowerShellTrainingCert" -CertStoreLocation "Cert:\CurrentUser\My" -KeyExportPolicy Exportable -KeySpec Signature -KeyLength 2048 -KeyAlgorithm RSA -HashAlgorithm SHA256
# New-SelfSignedCertificate: Cannot validate argument on parameter 'SubjectName'. The argument "CN=PowerShellTrainingCert" does not match the "^(CN|E|O|OU|C|ST|L):[^,:]+$" pattern. Supply an argument that matches "^(CN|E|O|OU|C|ST|L):[^,:]+$" and try the command again.

# SWITCH TO WINDOWS POWERSHELL FOR THE REMAINDER OF THIS DEMO

$certPath = "C:\demo-eid-msgraphps\PowerShellTrainingCert.cer"

Export-Certificate -Cert $cert -FilePath $certPath

# Register the application

# First, you're using the PowerShell SDK with delegated access, logging in as an administrator, and creating the app registration.
# Then, using that app registration, you're able to use the PowerShell SDK with app-only access, allowing for unattended scripts.
cd c:\demo-eid-msgraphps
code .\RegisterAppOnly.ps1

.\RegisterAppOnly.ps1 -AppName "Graph PSTraining Script" -CertPath $certPath

<# OUTPUT
Welcome To Microsoft Graph!
Certificate loaded
App registration created with app ID XXXXXXXX-959a-4b41-82fd-00ffce696971
Service principal created

Success

Please go to the following URL in your browser to provide admin consent
https://login.microsoftonline.com/XXXXXXXX-e799-4e34-9ce1-179f1f036bc3/adminconsent?client_id=XXXXXXXX-959a-4b41-82fd-00ffce696971

After providing admin consent, you can use the following values with Connect-MgGraph for app-only:
Connect-MgGraph -ClientId "XXXXXXXX-959a-4b41-82fd-00ffce696971" -TenantId "XXXXXXXX-e799-4e34-9ce1-179f1f036bc3" -CertificateName "CN=PowerShellTrainingCert"

Disconnected from Microsoft Graph
#>

# After granting admin consent, the browser will redirect back to http://localhost and display a Not Found error.
# This error can be ignored as long as the URL contains admin_consent=True.
# http://localhost/?admin_consent=True&tenant=XXXXXXXX-e799-4e34-9ce1-179f1f036bc3#

# Authenticate
Connect-MgGraph -ClientId "XXXXXXXX-959a-4b41-82fd-00ffce696971" -TenantId "XXXXXXXX-e799-4e34-9ce1-179f1f036bc3" -CertificateName "CN=PowerShellTrainingCert"

Get-MgContext

# Let's test the script

code .\GraphAppOnly.ps1

.\GraphAppOnly.ps1

#endregion