
# List all users
Get-MgUser

Get-MgUser -Filter "startswith(displayName,'a')" -OrderBy "displayName" -Top 1 -Count count -ConsistencyLevel eventual

Get-MgUser -Filter "endswith(mail,'g@DOMAINNAME.onmicrosoft.com')" -Sort "userPrincipalName" -CountVariable CountVar -ConsistencyLevel eventual 

Get-MgUser -Search '"displayName:al"' -Sort "displayName" -CountVariable CountVar -ConsistencyLevel eventual 

Get-MgUser -Search '"displayName:al" OR "displayName:di"' -Orderby displayname -CountVariable CountVar -ConsistencyLevel eventual 

# Use $filter to get users who are assigned a specific license
Get-MgUser -Select id,mail,assignedLicenses -Filter "assignedLicenses/any(u:u/skuId eq c42b9cae-ea4f-4ab7-9717-81576235ccac)"

# Get users including their last sign-in time

# Details for the signInActivity property require an Entra ID Premium P1/P2 license and the AuditLog.Read.All permission.

Get-MgUser -Property "displayName,userPrincipalName,signInActivity" 
Get-MgUser -Property displayName,userPrincipalName,signInActivity
# Where is signinActivity?
Get-MgUser -Property displayName,userPrincipalName,signInActivity | Get-Member 
Get-MgUser -Property displayName,userPrincipalName,signInActivity | Where-Object UserPrincipalName -eq 'aleksandar@DOMAINNAME.onmicrosoft.com'
$user = Get-MgUser -Property displayName,userPrincipalName,signInActivity | Where-Object UserPrincipalName -eq 'aleksandar@DOMAINNAME.onmicrosoft.com'
$user.signInActivity
$user.SignInActivity

$user.ToJsonString()
$user.ToJsonString() | ConvertFrom-Json | fl

Get-MgUser -Property displayName, userPrincipalName, signInActivity |
    ForEach-Object { $_.ToJsonString() | ConvertFrom-Json } | fl

# Let's compare Get-MgUser and Get-AzADUser
Get-MgUser | Select-Object Id, DisplayName, UserType -First 3
Get-MgUser -Select UserType | Select-Object Id, DisplayName, UserType -First 3
Get-MgUser -Select Id, DisplayName, UserType | Select-Object Id, DisplayName, UserType -First 3
Get-MgUser -Select Id, DisplayName, UserType | Select-Object Id, DisplayName, UserType, UserPrincipalName -First 3
$props = echo Id DisplayName UserType UserPrincipalName
Get-MgUser -Select $props | Select-Object $props -First 3

Get-AzADUser | Select-Object Id, DisplayName, UserType -First 3
Get-AzADUser -Select UserType | Select-Object Id, DisplayName, UserType -First 3
Get-AzADUser -Select UserType -AppendSelected | Select-Object Id, DisplayName, UserType, UserPrincipalName -First 3

$userId = 'DiegoS@DOMAINNAME.onmicrosoft.com'

# A UPN can also be used as -UserId.
Get-MgUser -UserId $userId

Get-MgUser -UserId $userId -Property displayName,givenName,postalCode,identities -ov diego 
$diego | gm
$diego.AdditionalProperties
$diego | Select-Object -Property displayName,givenName,postalCode,identities
$diego.Identities

$diego.ToJsonString() | ConvertFrom-Json | fl

# Create a new user

Get-Command New-MgUser -Syntax

$params = @{
	accountEnabled = $true
	displayName = "Adele Mance"
	mailNickname = "AdeleM"
	userPrincipalName = "AdeleM@DOMAINNAME.onmicrosoft.com"
	passwordProfile = @{
		forceChangePasswordNextSignIn = $true
		password = "XXXXXXXXXXXXXXXXXXXXXX"
	}
}

New-MgUser -BodyParameter $params

# Update the passwordProfile of a user to reset their password
$params = @{
	passwordProfile = @{
		forceChangePasswordNextSignIn = $true
		password = "YYYYYYYYYYYYYYYYYYYY"
	}
}

Update-MgUser -UserId 'AdeleM@DOMAINNAME.onmicrosoft.com' -BodyParameter $params

# Delete a user
# When deleted, user resources are moved to a temporary container and can be restored within 30 days.
# After that time, they are permanently deleted. 

Remove-MgUser -UserId $userId2 

# Change your own password
# Enable the user to update their password.
# Any user can update their password without belonging to any administrator role.
$params = @{
	currentPassword = "XXXXXXXXXXXXXXXXXXXXXX"
	newPassword = "YYYYYYYYYYYYYYYYYYYY"
}

# A UPN can also be used as -UserId.
Update-MgUserPassword -UserId $userId -BodyParameter $params

#region Org Hierarchy

# Get the direct reports of a user
Get-MgUserDirectReport -UserId $userId -ov directReport

$directReport | fl
$directReport.AdditionalProperties # a case-sensitive hash table
$directReport.tojsonstring() | ConvertFrom-Json

Get-MgUser -UserId $userId -ExpandProperty "directReports" -ov user_expand
$user_expand.DirectReports.AdditionalProperties

# Get the manager of a user
Get-MgUserManager -UserId $userId -ov manager
$manager.ToJsonString() | ConvertFrom-Json

# Get the manager of a user and expand the manager's manager
# Get manager chain up to the root level

# Get-MgUser: Parameter set cannot be resolved using the specified named parameters.
Get-MgUser -UserId 'testni@DOMAINNAME.onmicrosoft.com' -ExpandProperty "manager(`$levels=max;`$select=id,displayName)" -Property "id,displayName" -CountVariable CountVar -ConsistencyLevel eventual
# you expect to get transitive managers, but you get only direct manager
Get-MgUser -UserId 'testni@DOMAINNAME.onmicrosoft.com' -ExpandProperty "manager(`$levels=max;`$select=id,displayName)" -Property "id,displayName"

Get-MgUser -UserId 'testni@DOMAINNAME.onmicrosoft.com' -ExpandProperty "manager(`$select=id,displayName)" -Property "id,displayName" -ov managers
$managers.ToJsonString() | ConvertFrom-Json | fl

# returns transitive managers (with an advanced query)
Invoke-MgGraphRequest -Method GET -Uri "https://graph.microsoft.com/v1.0/users/testni@DOMAINNAME.onmicrosoft.com?`$expand=manager(`$levels=max;`$select=id,displayName)&`$select=id,displayName&`$count=true" -Headers @{ConsistencyLevel='eventual'} -ov response1

$response1.manager
$response1.manager.manager

# returns only direct manager (without an advanced query)
Invoke-MgGraphRequest -Method GET -Uri "https://graph.microsoft.com/v1.0/users/testni@DOMAINNAME.onmicrosoft.com?`$expand=manager(`$levels=max;`$select=id,displayName)&`$select=id,displayName" -ov response2

$response2.manager

# Assign a user's manager

# Navigation properties like manager cannot be used to update a user, and Update-MgUser will give you an error.

$AdeleM = Get-MgUser -UserId adelem@DOMAINNAME.onmicrosoft.com
$me = Get-MgUser -UserId aleksandar@DOMAINNAME.onmicrosoft.com

# The request body is a JSON object with an @odata.id parameter and the read URL for the user object to be assigned as a manager
$params = @{
	"@odata.id" = "https://graph.microsoft.com/v1.0/users/$($me.Id)"
}

Set-MgUserManagerByRef -UserId $AdeleM.Id -BodyParameter $params

Get-MgUserManager -UserId $AdeleM.Id -ov manager
$manager.ToJsonString() | ConvertFrom-Json

# Remove a user's manager
Remove-MgUserManagerByRef -UserId $AdeleM.Id

# Report all managers in the organization

Get-MgUser | ForEach-Object {$ht = @{}} { 

    $manager = Get-MgUserManager -UserId $_.Id -ea SilentlyContinue
    $ht = [ordered]@{
        DisplayName = $_.DisplayName
        UserPrincipalName = $_.UserPrincipalName
        Manager = ($?) ? "$($manager.AdditionalProperties.displayName) ($($manager.AdditionalProperties.userPrincipalName))" : "Without a manager" # displayName is case sensitive
    }
    [PSCustomObject]$ht 
 } | Sort-Object Manager | ft DisplayName, UserPrincipalName -GroupBy manager

#endregion



